@@ -1,3 +1,4 @@
+import enum
 import os
 import string
 
@@ -17,7 +18,7 @@ from easyshare.utils.types import to_int
 APP_NAME = "easyshare"
 APP_NAME_SERVER = "esd"
 APP_NAME_CLIENT = "es"
-APP_VERSION = "0.4"
+APP_VERSION = "0.5"
 
 APP_INFO = f"{APP_NAME} {APP_VERSION}"
 
@@ -60,15 +61,14 @@ DEFAULT_DISCOVER_TIMEOUT = 2    # sec
 BEST_BUFFER_SIZE = 4096
 
 
-def transfer_port(server_port: int):
-    return server_port + 1
+class TransferProtocol(enum.Enum):
+    TCP = "TCP"
+    UDP = "UDP"
 
 
-# =====================
-# ==== PYRO NETWORK ====
-# =====================
-
-ESD_PYRO_UID = "esd"
+class TransferDirection(enum.Enum):
+    IN = "IN"
+    OUT = "OUT"
 
 
 # =====================
