@@ -1,7 +1,7 @@
 import ssl
 
 from easyshare.sockets import SocketTcpAcceptor
-from easyshare.utils.types import str_to_bytes, bytes_to_str
+from easyshare.utils.types import stob, btos
 
 # Create cert and private key with:
 # openssl req -x509 -keyout privkey.pem -days 365 -nodes -out cert.pem
@@ -26,7 +26,7 @@ def main():
             print("Connection received from", endpoint)
             while True:
                 inraw = ns.recv(1024)
-                print("<< " + bytes_to_str(inraw))
+                print("<< " + btos(inraw))
 
 if __name__ == "__main__":
     main()
\ No newline at end of file
