@@ -11,12 +11,18 @@ from easyshare.helps.es import Es
 from easyshare.logging import get_logger
 from easyshare.res.helps import get_command_usage
 from easyshare.styling import enable_colors
-from easyshare.tracing import enable_tracing
+from easyshare.tracing import TRACING_NONE, TRACING_TEXT, set_tracing_level
 from easyshare.utils import abort, terminate
 from easyshare.utils.env import is_stdout_terminal, are_colors_supported
 from easyshare.utils.net import is_valid_port
 from easyshare.utils.obj import values
-from easyshare.utils.pyro import enable_pyro_logging
+
+
+# if __name__ == "__main__":
+
+# Call it now before get_logger for enable colors properly
+# and let logger be initialized with/without colors
+easyshare_setup()
 
 
 log = get_logger(__name__)
@@ -56,7 +62,8 @@ CLI_COMMANDS = [k for k in values(Commands) if k not in NON_CLI_COMMANDS]
 
 
 def main():
-    easyshare_setup()
+    # Already called
+    # easyshare_setup()
 
     # Parse arguments
     args = None
@@ -70,7 +77,6 @@ def main():
     # Eventually set verbosity before anything else
     # so that the rest of the startup (config parsing, ...)
     # can be logged
-    # Verbosity over VERBOSITY_MAX enables pyro logging too
     if args.has_option(Es.VERBOSE):
         log.set_verbosity(args.get_option_param(Es.VERBOSE,
                                                 default=logging.VERBOSITY_MAX))
@@ -86,8 +92,8 @@ def main():
     if Es.VERSION in args:
         terminate(APP_INFO)
 
-    verbosity = 0
-    tracing = 0
+    verbosity = logging.VERBOSITY_NONE
+    tracing = TRACING_NONE
     no_colors = False
     discover_port = DEFAULT_DISCOVER_PORT
     discover_timeout = DEFAULT_DISCOVER_TIMEOUT
@@ -98,11 +104,11 @@ def main():
 
     # Packet tracing
     if Es.TRACE in args:
-        # The param of -v is optional:
-        # if not specified the default is DEBUG
+        # The param of -t is optional:
+        # if not specified the default is TEXT
         tracing = args.get_option_param(
             Es.TRACE,
-            default=1
+            default=TRACING_TEXT
         )
 
     # Verbosity
@@ -145,10 +151,10 @@ def main():
 
     enable_colors(are_colors_supported() and not no_colors)
 
-    enable_tracing(tracing)
+    set_tracing_level(tracing)
+
     if verbosity:
         log.set_verbosity(verbosity)
-        enable_pyro_logging(verbosity > logging.VERBOSITY_MAX)
 
 
     # Initialize the client
