@@ -1,9 +1,11 @@
 import string
 from pathlib import Path
-from typing import Optional
+from typing import Optional, Any
 from easyshare.endpoint import Endpoint
 from easyshare.logging import get_logger
 from easyshare.protocol.types import SharingInfo, FTYPE_FILE, FTYPE_DIR, FileType, ftype
+from easyshare.sockets import SocketTcp
+from easyshare.streams import TcpStream
 from easyshare.utils.json import j
 from easyshare.utils.path import LocalPath
 from easyshare.utils.rand import randstring
@@ -19,15 +21,15 @@ log = get_logger(__name__)
 class ClientContext:
     """ Contains the server-side information kept for a connected client """
 
-    def __init__(self, endpoint: Endpoint):
-        # Actually the endpoint it's just the first of the endpoints the client
-        # can have due to different connections to server/sharing, transfer, rexec, ...
-        # (is the one that calls connect())
-        self.endpoint: Optional[Endpoint] = endpoint #
+    def __init__(self, sock: SocketTcp):
+        self.socket: SocketTcp = sock
+        self.stream = TcpStream(sock)
+        self.endpoint: Optional[Endpoint] = sock.remote_endpoint()
         self.tag = randstring(4, alphabet=string.ascii_lowercase) # not an unique id, just a tag
 
+
     def __str__(self):
-        return f"{self.endpoint} [{self.tag}]"
+        return f"{self.endpoint[0]}:{self.endpoint[1]} [{self.tag}]"
 
 # =============================================
 # ================== SHARING ==================
@@ -83,7 +85,7 @@ class Sharing:
             read_only=True if read_only else False,
         )
 
-        log.i("Created sharing: %s", j(sh.info()))
+        log.i("Created sharing: %s", j(sh._info_internal()))
 
         return sh
 
@@ -93,4 +95,13 @@ class Sharing:
             "name": self.name,
             "ftype": self.ftype,
             "read_only": self.read_only,
+        }
+
+    def _info_internal(self) -> SharingInfo:
+        """ Returns information ('SharingInfo') for this sharing + the internal path """
+        return {
+            "path": str(self.path),
+            "name": self.name,
+            "ftype": self.ftype,
+            "read_only": self.read_only,
         }
\ No newline at end of file
