@@ -2,18 +2,17 @@ import os
 from abc import abstractmethod, ABC
 from typing import List, Callable, Union, Optional, Dict, Type
 
-from easyshare.args import Option, PRESENCE_PARAM, INT_PARAM, NoPosArgsSpec, PosArgsSpec, VarArgsSpec
+from easyshare.args import Option, PRESENCE_PARAM, INT_PARAM, NoPosArgsSpec, PosArgsSpec, VarArgsSpec, STR_PARAM, \
+    StopParseArgsSpec
 from easyshare.es.ui import StyledString
 from easyshare.helps import CommandHelp, CommandOptionInfo
 from easyshare.logging import get_logger
 from easyshare.common import DIR_COLOR, FILE_COLOR
-from easyshare.protocol.services import FileInfo
 from easyshare.protocol.responses import is_data_response
-from easyshare.protocol.types import FTYPE_FILE, FTYPE_DIR
+from easyshare.protocol.types import FTYPE_FILE, FTYPE_DIR, FileInfo
 from easyshare.styling import fg
 from easyshare.utils.os import ls
 from easyshare.utils.path import LocalPath
-from easyshare.utils.str import rightof
 
 log = get_logger(__name__)
 
@@ -45,6 +44,9 @@ class Commands:
     LOCAL_LIST_DIRECTORY = "ls"
     LOCAL_LIST_DIRECTORY_ENHANCED = "l"
     LOCAL_TREE_DIRECTORY = "tree"
+    LOCAL_FIND = "find"
+    LOCAL_DISK_USAGE = "du"
+    LOCAL_FIND_SHORT = "f"
     LOCAL_CHANGE_DIRECTORY = "cd"
     LOCAL_CREATE_DIRECTORY = "mkdir"
     LOCAL_COPY = "cp"
@@ -59,6 +61,9 @@ class Commands:
     REMOTE_LIST_DIRECTORY = "rls"
     REMOTE_LIST_DIRECTORY_ENHANCED = "rl"
     REMOTE_TREE_DIRECTORY = "rtree"
+    REMOTE_FIND = "rfind"
+    REMOTE_DISK_USAGE = "rdu"
+    REMOTE_FIND_SHORT = "rf"
     REMOTE_CHANGE_DIRECTORY = "rcd"
     REMOTE_CREATE_DIRECTORY = "rmkdir"
     REMOTE_COPY = "rcp"
@@ -73,12 +78,13 @@ class Commands:
     SCAN_SHORT = "s"
 
     CONNECT = "connect"
+    CONNECT_SHORT = "c"
+
     DISCONNECT = "disconnect"
 
     OPEN = "open"
     OPEN_SHORT = "o"
     CLOSE = "close"
-    CLOSE_SHORT = "c"
 
     GET = "get"
     GET_SHORT = "g"
@@ -131,7 +137,7 @@ class SuggestionsIntent:
 class CommandInfo(CommandHelp, ABC):
     """ Provide full information of a command and the suggestions too """
     @classmethod
-    def suggestions(cls, token: str, line: str, client) -> Optional[SuggestionsIntent]:
+    def suggestions(cls, token: str, client) -> Optional[SuggestionsIntent]:
         options = cls.options()
 
         if not options:
@@ -166,6 +172,26 @@ class CommandInfo(CommandHelp, ABC):
                                  max_columns=1)
 
 
+# dummy
+class CommandAlias(CommandInfo):
+    @classmethod
+    def synopsis(cls) -> str:
+        pass
+
+    @classmethod
+    def short_description(cls) -> str:
+        pass
+
+    @classmethod
+    def long_description(cls) -> str:
+        pass
+
+    @classmethod
+    def name(cls) -> str:
+        pass
+
+
+
 class FilesSuggestionsCommandInfo(CommandInfo):
     """ Base suggestions provided of list of files/directory """
     @classmethod
@@ -176,20 +202,20 @@ class FilesSuggestionsCommandInfo(CommandInfo):
 
     @classmethod
     @abstractmethod
-    def _provide_file_info_list(cls, token: str, line: str, client) -> List[FileInfo]:
+    def _provide_file_info_list(cls, token: str, client) -> List[FileInfo]:
         """ The file list to suggest, after being filtered by '_file_info_filter' """
         pass
 
     @classmethod
-    def suggestions(cls, token: str, line: str, client) -> Optional[SuggestionsIntent]:
+    def suggestions(cls, token: str, client) -> Optional[SuggestionsIntent]:
         log.d("Providing files listing suggestions")
 
-        suggestions_intent = super().suggestions(token, line, client)
+        suggestions_intent = super().suggestions(token, client)
         if suggestions_intent:
             return suggestions_intent
 
         suggestions = []
-        for finfo in cls._provide_file_info_list(token, line, client):
+        for finfo in cls._provide_file_info_list(token, client):
             log.d("Suggestion finfo: %s", finfo)
 
             fname = finfo.get("name")
@@ -199,21 +225,25 @@ class FilesSuggestionsCommandInfo(CommandInfo):
                 continue
 
             _, fname_tail = os.path.split(fname)
+            token_head, token_tail = os.path.split(token)
 
-            if not fname_tail.lower().startswith(token.lower()):
+            if not fname_tail.lower().startswith(token_tail.lower()):
+                log.d(f"Does not match: '{fname_tail.lower()}' and '{token_tail.lower()}'")
                 continue
 
+            model = os.path.join(token_head, fname_tail)
+            view = fname_tail
+
             if finfo.get("ftype") == FTYPE_DIR:
                 # Append a dir, with a trailing / so that the next
                 # suggestion can continue to traverse the file system
-                ff = fname_tail + "/"
-                suggestions.append(StyledString(ff, fg(ff, color=DIR_COLOR)))
+                suggestions.append(StyledString(model + "/", fg(view + "/", color=DIR_COLOR)))
             else:
                 # Append a file, with a trailing space since there
                 # is no need to traverse the file system
-                ff = fname_tail
-                suggestions.append(StyledString(ff, fg(ff, color=FILE_COLOR)))
+                suggestions.append(StyledString(model, fg(view, color=FILE_COLOR)))
 
+        log.d(f"There will be {len(suggestions)} suggestions")
         return SuggestionsIntent(suggestions,
                                  completion=True,
                                  space_after_completion=lambda s: not s.endswith("/"))
@@ -228,41 +258,42 @@ class FilesSuggestionsCommandInfo(CommandInfo):
 class LocalFilesSuggestionsCommandInfo(FilesSuggestionsCommandInfo, ABC):
     """ Files suggestions provider of local files """
     @classmethod
-    def _provide_file_info_list(cls, token: str, line: str, client) -> List[FileInfo]:
-        log.d("List on token = '%s', line = '%s'", token, line)
+    def _provide_file_info_list(cls, token: str, client) -> List[FileInfo]:
+        log.d("List on token = '%s'", token)
         # token contains only the last part after a /
         # e.g. /tmp/something => something
         # we have to use all the path (line)
 
         # Take the part after the last space
-        pattern = rightof(line, " ", from_end=True)
 
         # Take the parent
         # path.parent can't be used unconditionally since it returns
         # the parent dir even if "pattern" ends with a os.path.sep
-        path = LocalPath(pattern)
-        if not pattern.endswith(os.path.sep):
+        path = LocalPath(token)
+        if not token.endswith(os.path.sep):
             path = path.parent
 
         log.i("ls-ing for suggestions on '%s'", path)
-        return ls(path)
+        return ls(path, details=False)
 
 
 class RemoteFilesSuggestionsCommandInfo(FilesSuggestionsCommandInfo, ABC):
     """ Files suggestions provider of remote files (performs an rls) """
 
     @classmethod
-    def _provide_file_info_list(cls, token: str, line: str, client) -> List[FileInfo]:
+    def _provide_file_info_list(cls, token: str, client) -> List[FileInfo]:
         if not client or not client.is_connected_to_sharing():
             log.w("Cannot list suggestions on a non connected es")
             return []
 
-        log.i("List remotely on token = '%s', line = '%s'", token, line)
-        pattern = rightof(line, " ", from_end=True)
-        path_dir, path_trail = os.path.split(pattern)
+        log.i("List remotely on token = '%s'", token)
+        path_dir, path_trail = os.path.split(token)
 
-        log.i("rls-ing on %s", pattern)
-        resp = client.sharing_connection.rls(sort_by=["name"], path=path_dir)
+        log.i("rls-ing on %s", token)
+        # rls the remote, use -a if we the user is submitting a dot
+
+        resp = client.connection.rls(sort_by=["name"], hidden=token.startswith("."),
+                                     path=path_dir)
 
         if not is_data_response(resp):
             log.w("Unable to retrieve a valid response for rls")
@@ -386,7 +417,7 @@ Available commands are:
 {comms}"""
 
     @classmethod
-    def suggestions(cls, token: str, line: str, client) -> Optional[SuggestionsIntent]:
+    def suggestions(cls, token: str, client) -> Optional[SuggestionsIntent]:
         log.d("Providing commands suggestions")
 
         suggestions = [StyledString(comm)
@@ -426,8 +457,10 @@ Open connections are automatically closed."""
 # ============ TRACE ================
 
 class Trace(CommandInfo):
-    T0 = (["0"], "enable packet tracing")
-    T1 = (["1"], "disable packet tracing")
+    T0 = (["0"], "disabled")
+    T1 = (["1"], "text")
+    T2 = (["2"], "binary payloads")
+    T3 = (["3"], "binary all")
 
     @classmethod
     def name(cls):
@@ -441,72 +474,82 @@ class Trace(CommandInfo):
     def synopsis(cls):
         return """\
 trace <A> # just for alignment
-<b>trace</b> [<u>0</u> | <u>1</u>]
-<b>t</b>     <a>[<u>0</u> | <u>1</u>]</a>"""
+<b>trace</b> <a>[<u>LEVEL</u>]</a>
+<b>t</b>     <a>[<u>LEVEL</u>]</a>"""
 
     @classmethod
     def long_description(cls):
         return """\
-Show (1) or hide (0) the packets sent and received to and from the server for any operation.
+Change the tracing level to <u>LEVEL</u> (default is <u>0</u>, which is disabled).
+When tracing is enabled, packets sent and received to and from the server for any operation are shown.
+
+The allowed values of <u>LEVEL</u> are:
+<u>0</u>        disabled (default)
+<u>1</u>        text
+<u>2</u>        binary payloads
+<u>3</u>        binary all (payload + header)
+
+If no argument is given, increase the tracing level or resets \
+it to <u>0</u> if it exceeds the maximum."""
 
-If no argument is given, toggle the packet tracing mode."""
 
     @classmethod
     def examples(cls):
-        return """\
-Usage example:
-
-<b>/tmp</b> <b>t</b>
-Tracing = 1 (enabled)
-<b>/tmp</b> o temp
->> <broadcast>:12019
->>   DISCOVER (58140)
-
-<< 192.168.1.105:57300
-<<   DISCOVER
-{
-<i+2>"success": true,</i>
-<i+2>"data": {</i>
-<i+4>"name": "alice-arch",</i>
-<i+4>"sharings": [</i>
-<i+6>{</i>
-<i+8>"name": "temp",</i>
-<i+8>"ftype": "dir",</i>
-<i+8>"read_only": false</i>
-<i+6>}</i>
-<i+4>],</i>
-<i+4>"ssl": false,</i>
-<i+4>"auth": false,</i>
-<i+4>"ip": "192.168.1.105",</i>
-<i+4>"port": 12020,</i>
-<i+4>"discoverable": true,</i>
-<i+4>"discover_port": 12019</i>
-<i+2>}</i>
-}
-
->> 192.168.1.105:12020 (stefano-arch)
->>   connect (password=None)
-
-<< 192.168.1.105:12020 (stefano-arch)
-<<   connect
-{
-<i+2>"success": true</i>
-}
-
->> 192.168.1.105:12020 (stefano-arch)
->>   open ("temp")
-
-<< 192.168.1.105:12020 (stefano-arch)
-<<   open
-{
-<i+2>"success": true,</i>
-<i+2>"data": "ebc40f8680d448dcad5c98291b720e37"</i>
-}
-
-<b>alice-arch.temp:/</b> - <b>/tmp></b>"""
+        return "TO DO"
+#         return """\
+# Usage example:
+#
+# <b>/tmp</b> <b>t</b>
+# Tracing = 1 (enabled)
+# <b>/tmp</b> o temp
+# >> <broadcast>:12019
+# >>   DISCOVER (58140)
+#
+# << 192.168.1.105:57300
+# <<   DISCOVER
+# {
+# <i+2>"success": true,</i>
+# <i+2>"data": {</i>
+# <i+4>"name": "alice-arch",</i>
+# <i+4>"sharings": [</i>
+# <i+6>{</i>
+# <i+8>"name": "temp",</i>
+# <i+8>"ftype": "dir",</i>
+# <i+8>"read_only": false</i>
+# <i+6>}</i>
+# <i+4>],</i>
+# <i+4>"ssl": false,</i>
+# <i+4>"auth": false,</i>
+# <i+4>"ip": "192.168.1.105",</i>
+# <i+4>"port": 12020,</i>
+# <i+4>"discoverable": true,</i>
+# <i+4>"discover_port": 12019</i>
+# <i+2>}</i>
+# }
+#
+# >> 192.168.1.105:12020 (stefano-arch)
+# >>   connect (password=None)
+#
+# << 192.168.1.105:12020 (stefano-arch)
+# <<   connect
+# {
+# <i+2>"success": true</i>
+# }
+#
+# >> 192.168.1.105:12020 (stefano-arch)
+# >>   open ("temp")
+#
+# << 192.168.1.105:12020 (stefano-arch)
+# <<   open
+# {
+# <i+2>"success": true,</i>
+# <i+2>"data": "ebc40f8680d448dcad5c98291b720e37"</i>
+# }
+#
+# <b>alice-arch.temp:/</b> - <b>/tmp></b>"""
 
     @classmethod
-    def suggestions(cls, token: str, line: str, client) -> Optional[SuggestionsIntent]:
+    def suggestions(cls, token: str, client) -> Optional[SuggestionsIntent]:
         return SuggestionsIntent(
             [StyledString(info.to_str(justification=15 + 6))
              for info in [
@@ -552,7 +595,7 @@ Change the verbosity level to <u>LEVEL</u> (default is <u>0</u>, which is disabl
 
 The messages are written to stdout.
 
-The allowed value of <u>LEVEL</u> are:
+The allowed values of <u>LEVEL</u> are:
 <u>0</u>        disabled (default)
 <u>1</u>        errors
 <u>2</u>        warnings
@@ -564,7 +607,7 @@ If no argument is given, increase the verbosity or resets \
 it to <u>0</u> if it exceeds the maximum."""
 
     @classmethod
-    def suggestions(cls, token: str, line: str, client) -> Optional[SuggestionsIntent]:
+    def suggestions(cls, token: str, client) -> Optional[SuggestionsIntent]:
         return SuggestionsIntent(
             [StyledString(info.to_str(justification=15 + 6))
              for info in [
@@ -584,7 +627,9 @@ it to <u>0</u> if it exceeds the maximum."""
 # ============ xPWD ================
 
 
-class Pwd(CommandInfo):
+class Pwd(CommandInfo, PosArgsSpec):
+    def __init__(self):
+        super().__init__(0, 0)
 
     @classmethod
     def name(cls):
@@ -610,7 +655,9 @@ The local working directory can be changed with the command <b>cd</b>."""
         return """Type "<b>help rpwd</b>" for the remote analogous."""
 
 
-class Rpwd(CommandInfo):
+class Rpwd(CommandInfo, PosArgsSpec):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory, 0)
 
     @classmethod
     def name(cls):
@@ -639,6 +686,7 @@ The remote working directory can be changed with the command <b>rcd</b>."""
 
 
 class BaseLsCommandInfo(CommandInfo, ABC, PosArgsSpec):
+
     SORT_BY_SIZE = ["-s", "--sort-size"]
     REVERSE = ["-r", "--reverse"]
     GROUP = ["-g", "--group"]
@@ -670,8 +718,8 @@ class BaseLsCommandInfo(CommandInfo, ABC, PosArgsSpec):
         ]
 
 class Ls(LocalAllFilesSuggestionsCommandInfo, BaseLsCommandInfo):
-    def __init__(self, mandatory: int):
-        super().__init__(mandatory, 1)
+    def __init__(self):
+        super().__init__(0, 1)
 
     @classmethod
     def name(cls):
@@ -728,13 +776,19 @@ List content of the remote <u>DIR</u> or the current remote directory if no <u>D
 # ============ xL ================
 
 # noinspection PyAbstractClass
-class L(CommandInfo):
+class L(CommandAlias, PosArgsSpec):
+    def __init__(self):
+        super().__init__(0, 1)
+
     @classmethod
     def custom(cls):
         return "alias for ls -la"
 
 # noinspection PyAbstractClass
-class Rl(CommandInfo):
+class Rl(CommandAlias, PosArgsSpec):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory, 1)
+
     @classmethod
     def custom(cls):
         return "alias for rls -la"
@@ -778,8 +832,8 @@ class BaseTreeCommandInfo(CommandInfo, ABC, PosArgsSpec):
 
 
 class Tree(BaseTreeCommandInfo, LocalAllFilesSuggestionsCommandInfo):
-    def __init__(self, mandatory: int):
-        super().__init__(mandatory, 1)
+    def __init__(self):
+        super().__init__(0, 1)
 
     @classmethod
     def name(cls):
@@ -855,10 +909,278 @@ remote directory if no <u>DIR</u> is specified"""
     def see_also(cls):
         return """Type "<b>help tree</b>" for the local analogous."""
 
+
+# ============ XFIND ==============
+
+
+class BaseFindCommandInfo(CommandInfo, ABC, PosArgsSpec):
+    NAME = ["-n", "--name"]
+    REGEX = ["-r", "--regex"]
+    CASE_INSENSITIVE = ["-i", "--insensitive"]
+    TYPE = ["-t", "--type"]
+    SHOW_DETAILS = ["-l"]
+
+    def options_spec(self) -> Optional[List[Option]]:
+        return [
+            (self.NAME, STR_PARAM),
+            (self.REGEX, STR_PARAM),
+            (self.CASE_INSENSITIVE, PRESENCE_PARAM),
+            (self.TYPE, STR_PARAM),
+            (self.SHOW_DETAILS, PRESENCE_PARAM),
+        ]
+
+    @classmethod
+    def options(cls) -> List[CommandOptionInfo]:
+        return [
+            CommandOptionInfo(cls.NAME, "filter by filename",
+                              params=["pattern"]),
+            CommandOptionInfo(cls.REGEX, "filter by filename using regular expression",
+                              params=["pattern"]),
+            CommandOptionInfo(cls.CASE_INSENSITIVE, "make filename filtering case insensitive"),
+            CommandOptionInfo(cls.TYPE, "filter by file type",
+                              params=["ftype"]),
+            CommandOptionInfo(cls.SHOW_DETAILS, "show more details"),
+        ]
+
+class Find(LocalAllFilesSuggestionsCommandInfo, BaseFindCommandInfo):
+    def __init__(self):
+        super().__init__(0, 1)
+
+    @classmethod
+    def name(cls):
+        return "find"
+
+    @classmethod
+    def short_description(cls):
+        return "search for local files"
+
+    @classmethod
+    def synopsis(cls):
+        return """\
+find <A> # just for alignment
+<b>find</b> [<u>OPTION</u>]... [<u>DIR</u>]</a>
+<b>f</b> [<u>OPTION</u>]... [<u>DIR</u>]</a>"""
+
+    @classmethod
+    def long_description(cls):
+        return """\
+Search for files and directories based on the given filters the local <u>DIR</u>, \
+or the current local directory if no <u>DIR</u> is specified.
+
+Each result of <b>find</b> is memorized and can be used further with any command \
+that accept local paths by specifying the identifier shown by <b>find</b>.
+For instance, you can search a file by name \
+(i.e. <b>find</b> <b>-n</b> <u>usefulname</u>) and then perform a local action over it \
+(e.g. <b>rm</b> $a1) or even a transfer action (e.g. <b>put</b> $a1)."""
+
+    @classmethod
+    def see_also(cls):
+        return """Type "<b>help rfind</b>" for the remote analogous."""
+
+    @classmethod
+    def examples(cls):
+        return """\
+Usage example:
+   <a>
+1. List current local directory (no filters)</a>
+<b>/tmp></b> <b>find</b>
+$a1 FILE2
+$a2 dir1
+$a3 dir1/file
+$a4 dir2
+$a5 file1
+   <a>
+2. List local directory (no filters)</a>
+<b>/tmp></b> <b>find</b> dir1
+$b3 dir1/file
+   <a>
+3. Finding by name</a>
+<b>/tmp></b> <b>find</b> <b>-n</b> <u>file</u>
+$c1 dir1/file
+$c2 file1
+   <a>
+4. Finding by name, case insensitive</a>
+<b>/tmp></b> <b>find</b> <b>-in</b> <u>file</u>
+$d1 FILE2
+$d2 dir1/file
+$d3 file1
+   <a>
+5. Finding by regex</a>
+<b>/tmp></b> <b>find</b> <b>-r</b> <u>file[0-9]</u>
+$e1 file1
+   <a>
+6. Finding by regex, case insensitive</a>
+<b>/tmp></b> <b>find</b> <b>-ir</b> <u>file[0-9]</u>
+$f1 FILE2
+$f2 file1   
+    <a>
+7. Finding by name, files only</a>
+<b>/tmp></b> <b>find</b> <b>-in</b> <u>dir1</u> <b>-t</b> <u>f</u>
+$g1 dir1/file"""
+
+
+
+class Rfind(RemoteAllFilesSuggestionsCommandInfo, BaseFindCommandInfo, FastSharingConnectionCommandInfo):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory, 1)
+
+    @classmethod
+    def name(cls):
+        return "rfind"
+
+    @classmethod
+    def short_description(cls):
+        return "search for remote files"
+
+    @classmethod
+    def synopsis(cls):
+        return """\
+rfind <A> # just for alignment
+<b>rfind</b> [<u>OPTION</u>]... [<u>DIR</u>]</a>
+<b>rf</b> [<u>OPTION</u>]... [<u>DIR</u>]</a>"""
+
+    @classmethod
+    def long_description(cls):
+        return """\
+Search for files and directories based on the given filters the remote <u>DIR</u>, \
+or the current remote directory if no <u>DIR</u> is specified.
+
+Each result of <b>rfind</b> is memorized and can be used further with any command \
+that accept local paths by specifying the identifier shown by <b>find</b>.
+For instance, you can search a file by name \
+(i.e. <b>find</b> <b>-n</b> <u>usefulname</u>) and then perform a local action over it \
+(e.g. <b>rrm</b> $a1) or even a transfer action (e.g. <b>get</b> $a1)."""
+
+    @classmethod
+    def see_also(cls):
+        return """Type "<b>help find</b>" for the local analogous."""
+
+    @classmethod
+    def examples(cls):
+        return """\
+Usage example:
+   <a>
+1. List current remote directory (no filters)</a>
+<b>bob-debian.shared:/</b> - <b>/tmp></b> <b>find</b>
+$a1 FILE2
+$a2 dir1
+$a3 dir1/file
+$a4 dir2
+$a5 file1
+   <a>
+2. List remote directory (no filters)</a>
+<b>bob-debian.shared:/</b> - <b>/tmp></b> <b>find</b> dir1
+$b3 dir1/file
+   <a>
+3. Finding by name</a>
+<b>bob-debian.shared:/</b> - <b>/tmp></b> <b>find</b> <b>-n</b> <u>file</u>
+$c1 dir1/file
+$c2 file1
+   <a>
+4. Finding by name, case insensitive</a>
+<b>bob-debian.shared:/</b> - <b>/tmp></b> <b>find</b> <b>-in</b> <u>file</u>
+$d1 FILE2
+$d2 dir1/file
+$d3 file1
+   <a>
+5. Finding by regex</a>
+<b>bob-debian.shared:/</b> - <b>/tmp></b> <b>find</b> <b>-r</b> <u>file[0-9]</u>
+$e1 file1
+   <a>
+6. Finding by regex, case insensitive</a>
+<b>bob-debian.shared:/</b> - <b>/tmp></b> <b>find</b> <b>-ir</b> <u>file[0-9]</u>
+$f1 FILE2
+$f2 file1   
+    <a>
+7. Finding by name, files only</a>
+<b>bob-debian.shared:/</b> - <b>/tmp></b> <b>find</b> <b>-in</b> <u>dir1</u> <b>-t</b> <u>f</u>
+$g1 dir1/file"""
+
+
+
+# ============ xDU ================
+
+
+class BaseDuCommandInfo(CommandInfo, ABC, PosArgsSpec):
+    HUMAN = ["-h", "--human"]
+
+    def options_spec(self) -> Optional[List[Option]]:
+        return [
+            (self.HUMAN, PRESENCE_PARAM),
+        ]
+
+    @classmethod
+    def options(cls) -> List[CommandOptionInfo]:
+        return [
+            CommandOptionInfo(cls.HUMAN, "print size in human readable format (e.g. 17K)")
+        ]
+
+
+class Du(LocalAllFilesSuggestionsCommandInfo, BaseDuCommandInfo):
+    def __init__(self):
+        super().__init__(0, 1)
+
+    @classmethod
+    def name(cls):
+        return "du"
+
+    @classmethod
+    def short_description(cls):
+        return "estimate disk usage of local files"
+
+    @classmethod
+    def synopsis(cls):
+        return """\
+du <A> # just for alignment
+<b>du</b> <u>FILE</u></a>"""
+
+    @classmethod
+    def long_description(cls):
+        return """\
+Estimate the disk usage of <u>FILE</u> (which could be either a file or a directory).
+If <u>FILE</u> is not specified, the disk usage of the current local directory is estimated instead."""
+
+    @classmethod
+    def see_also(cls):
+        return """Type "<b>help rdu</b>" for the remote analogous."""
+
+
+class Rdu(RemoteAllFilesSuggestionsCommandInfo, BaseDuCommandInfo, FastSharingConnectionCommandInfo):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory, 1)
+
+    @classmethod
+    def name(cls):
+        return "rdu"
+
+    @classmethod
+    def short_description(cls):
+        return "estimate disk usage of remote files"
+
+    @classmethod
+    def synopsis(cls):
+        return """\
+rdu <A> # just for alignment
+<b>rdu</b> [<u>FILE</u>]</a>"""
+
+    @classmethod
+    def long_description(cls):
+        return """\
+Estimate the disk usage of <u>FILE</u> (which could be either a file or a directory).
+If <u>FILE</u> is not specified, the disk usage of the current remote directory is estimated instead."""
+
+    @classmethod
+    def see_also(cls):
+        return """Type "<b>help du</b>" for the remote analogous."""
+
+
+
 # ============ xCD ================
 
 
-class Cd(LocalDirsOnlySuggestionsCommandInfo):
+class Cd(LocalDirsOnlySuggestionsCommandInfo, PosArgsSpec):
+    def __init__(self):
+        super().__init__(0, 1)
 
     @classmethod
     def name(cls):
@@ -885,7 +1207,9 @@ directory if <u>DIR</u> is not specified."""
         return """Type "<b>help rcd</b>" for the remote analogous."""
 
 
-class Rcd(RemoteDirsOnlySuggestionsCommandInfo):
+class Rcd(RemoteDirsOnlySuggestionsCommandInfo, PosArgsSpec):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory, 1)
 
     @classmethod
     def name(cls):
@@ -925,7 +1249,9 @@ Usage example:
 # ============ xMKDIR ================
 
 
-class Mkdir(LocalDirsOnlySuggestionsCommandInfo):
+class Mkdir(LocalDirsOnlySuggestionsCommandInfo, PosArgsSpec):
+    def __init__(self):
+        super().__init__(1, 0)
 
     @classmethod
     def name(cls):
@@ -955,7 +1281,9 @@ If <u>DIR</u> already exists, it does nothing."""
         return """Type "<b>help rmkdir</b>" for the remote analogous."""
 
 
-class Rmkdir(FastSharingConnectionCommandInfo, RemoteDirsOnlySuggestionsCommandInfo):
+class Rmkdir(FastSharingConnectionCommandInfo, RemoteDirsOnlySuggestionsCommandInfo, PosArgsSpec):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory, 0)
 
     @classmethod
     def name(cls):
@@ -1001,7 +1329,9 @@ Usage example:
 # ============ xCP ================
 
 
-class Cp(LocalAllFilesSuggestionsCommandInfo):
+class Cp(LocalAllFilesSuggestionsCommandInfo, VarArgsSpec):
+    def __init__(self):
+        super().__init__(2)
 
     @classmethod
     def name(cls):
@@ -1039,7 +1369,9 @@ be an existing directory and <u>SOURCE</u>s will be copied into it."""
         return """Type "<b>help rcp</b>" for the remote analogous."""
 
 
-class Rcp(FastSharingConnectionCommandInfo, RemoteAllFilesSuggestionsCommandInfo):
+class Rcp(FastSharingConnectionCommandInfo, RemoteAllFilesSuggestionsCommandInfo, VarArgsSpec):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory)
 
     @classmethod
     def name(cls):
@@ -1110,7 +1442,9 @@ f1      f2
 # ============ xMV ================
 
 
-class Mv(LocalAllFilesSuggestionsCommandInfo):
+class Mv(LocalAllFilesSuggestionsCommandInfo, VarArgsSpec):
+    def __init__(self):
+        super().__init__(2)
 
     @classmethod
     def name(cls):
@@ -1150,7 +1484,9 @@ be an existing directory and <u>SOURCE</u>s will be moved into it."""
 
 
 
-class Rmv(FastSharingConnectionCommandInfo, RemoteAllFilesSuggestionsCommandInfo):
+class Rmv(FastSharingConnectionCommandInfo, RemoteAllFilesSuggestionsCommandInfo, VarArgsSpec):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory)
 
     @classmethod
     def name(cls):
@@ -1217,7 +1553,9 @@ f2
 # ============ xRM ================
 
 
-class Rm(LocalAllFilesSuggestionsCommandInfo):
+class Rm(LocalAllFilesSuggestionsCommandInfo, VarArgsSpec):
+    def __init__(self):
+        super().__init__(1)
 
     @classmethod
     def name(cls):
@@ -1250,7 +1588,9 @@ This commands never prompts: essentially acts like unix's rm -rf."""
 
 
 
-class Rrm(FastSharingConnectionCommandInfo, RemoteAllFilesSuggestionsCommandInfo):
+class Rrm(FastSharingConnectionCommandInfo, RemoteAllFilesSuggestionsCommandInfo, VarArgsSpec):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory)
 
     @classmethod
     def name(cls):
@@ -1309,8 +1649,7 @@ f1
 # ============ xEXEC ================
 
 
-class Exec(LocalAllFilesSuggestionsCommandInfo):
-
+class Exec(LocalAllFilesSuggestionsCommandInfo, StopParseArgsSpec):
     @classmethod
     def name(cls):
         return "exec"
@@ -1360,7 +1699,9 @@ hello"""
         return """Type "<b>help rexec</b>" for the remote analogous."""
 
 
-class Rexec(FastServerConnectionCommandInfo, RemoteAllFilesSuggestionsCommandInfo):
+class Rexec(FastServerConnectionCommandInfo, RemoteAllFilesSuggestionsCommandInfo, StopParseArgsSpec):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory)
 
     @classmethod
     def name(cls):
@@ -1423,7 +1764,9 @@ hello"""
 # ============ xSHELL ===============
 
 
-class Shell(CommandInfo):
+class Shell(CommandInfo, PosArgsSpec):
+    def __init__(self):
+        super().__init__(0)
 
     @classmethod
     def name(cls):
@@ -1453,7 +1796,9 @@ Currently supported only for Unix."""
         return """Type "<b>help rshell</b>" for the remote analogous."""
 
 
-class Rshell(CommandInfo):
+class Rshell(CommandInfo, PosArgsSpec):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory)
 
     @classmethod
     def name(cls):
@@ -1554,7 +1899,9 @@ Usage example:
 # ============ CONNECT ================
 
 
-class Connect(CommandInfo):
+class Connect(CommandInfo, PosArgsSpec):
+    def __init__(self):
+        super().__init__(1)
 
     @classmethod
     def name(cls):
@@ -1644,7 +1991,9 @@ DIRECTORIES
 # ============ DISCONNECT ================
 
 
-class Disconnect(CommandInfo):
+class Disconnect(CommandInfo, PosArgsSpec):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory)
 
     @classmethod
     def name(cls):
@@ -1687,7 +2036,9 @@ Usage example:
 # ============ OPEN ================
 
 
-class Open(CommandInfo):
+class Open(CommandInfo, PosArgsSpec):
+    def __init__(self):
+        super().__init__(1)
 
     @classmethod
     def name(cls):
@@ -1801,7 +2152,9 @@ f1      f2
 # ============ CLOSE ================
 
 
-class Close(CommandInfo):
+class Close(CommandInfo, PosArgsSpec):
+    def __init__(self, mandatory: int):
+        super().__init__(mandatory)
 
     @classmethod
     def name(cls):
@@ -1850,19 +2203,34 @@ Usage example:
 
 
 class Get(RemoteAllFilesSuggestionsCommandInfo, VarArgsSpec):
-    OVERWRITE_YES = ["-y", "--yes"]
-    OVERWRITE_NO = ["-n", "--no"]
-    OVERWRITE_NEWER = ["-N", "--newer"]
+    OVERWRITE_YES = ["-y", "--overwrite-yes"]
+    OVERWRITE_NO = ["-n", "--overwrite-no"]
+    OVERWRITE_NEWER = ["-N", "--overwrite-newer"]
+    OVERWRITE_DIFF_SIZE = ["-S", "--overwrite-diff-size"]
+    PREVIEW = ["-p", "--preview"]
     CHECK = ["-c", "--check"]
     QUIET = ["-q", "--quiet"]
+    NO_HIDDEN = ["-h", "--no-hidden"]
+    SYNC = ["-s", "--sync"]
+
+    # Secret params
+    MMAP = ["--mmap"]
+    CHUNK_SIZE = ["--chunk-size"]
 
     def options_spec(self) -> Optional[List[Option]]:
         return [
             (self.OVERWRITE_YES, PRESENCE_PARAM),
             (self.OVERWRITE_NO, PRESENCE_PARAM),
             (self.OVERWRITE_NEWER, PRESENCE_PARAM),
+            (self.OVERWRITE_DIFF_SIZE, PRESENCE_PARAM),
+            (self.PREVIEW, PRESENCE_PARAM),
             (self.CHECK, PRESENCE_PARAM),
             (self.QUIET, PRESENCE_PARAM),
+            (self.NO_HIDDEN, PRESENCE_PARAM),
+            (self.SYNC, PRESENCE_PARAM),
+
+            (self.MMAP, INT_PARAM),
+            (self.CHUNK_SIZE, INT_PARAM),
         ]
 
     @classmethod
@@ -1928,7 +2296,8 @@ Directories are automatically downloaded recursively.
 
 If a remote file has the same name of a local file, you will be asked \
 whether overwrite it or not. The default overwrite behaviour can be specified \
-with the options <b>-y</b> (yes), <b>-n</b> (no), <b>N</b> (newer)."""
+with the options <b>-y</b> (yes), <b>-n</b> (no), <b>-N</b> (overwrite if newer) 
+and <b>-S</b> (overwrite if size is different)."""
 
     @classmethod
     def options(cls) -> List[CommandOptionInfo]:
@@ -1936,8 +2305,12 @@ with the options <b>-y</b> (yes), <b>-n</b> (no), <b>N</b> (newer)."""
             CommandOptionInfo(cls.OVERWRITE_YES, "always overwrite files"),
             CommandOptionInfo(cls.OVERWRITE_NO, "never overwrite files"),
             CommandOptionInfo(cls.OVERWRITE_NEWER, "overwrite files only if newer"),
+            CommandOptionInfo(cls.OVERWRITE_DIFF_SIZE, "overwrite files only if size is different"),
+            CommandOptionInfo(cls.PREVIEW, "do not transfer, just show a preview of what will happen"),
             CommandOptionInfo(cls.CHECK, "performs a check of files consistency"),
             CommandOptionInfo(cls.QUIET, "doesn't show progress"),
+            CommandOptionInfo(cls.NO_HIDDEN, "doesn't copy hidden files"),
+            CommandOptionInfo(cls.SYNC, "synchronize (same as -N but remove old files)"),
         ]
 
     @classmethod
@@ -2045,19 +2418,35 @@ Avg. speed   1MB/s
 
 
 class Put(LocalAllFilesSuggestionsCommandInfo, VarArgsSpec):
-    OVERWRITE_YES = ["-y", "--yes"]
-    OVERWRITE_NO = ["-n", "--no"]
-    OVERWRITE_NEWER = ["-N", "--newer"]
+    OVERWRITE_YES = ["-y", "--overwrite-yes"]
+    OVERWRITE_NO = ["-n", "--overwrite-no"]
+    OVERWRITE_NEWER = ["-N", "--overwrite-newer"]
+    OVERWRITE_DIFF_SIZE = ["-S", "--overwrite-diff-size"]
+    PREVIEW = ["-p", "--preview"]
     CHECK = ["-c", "--check"]
     QUIET = ["-q", "--quiet"]
+    NO_HIDDEN = ["-h", "--no-hidden"]
+    SYNC = ["-s", "--sync"]
+
+
+    # Secret params
+    MMAP = ["--mmap"]
+    CHUNK_SIZE = ["--chunk-size"]
 
     def options_spec(self) -> Optional[List[Option]]:
         return [
             (self.OVERWRITE_YES, PRESENCE_PARAM),
             (self.OVERWRITE_NO, PRESENCE_PARAM),
             (self.OVERWRITE_NEWER, PRESENCE_PARAM),
+            (self.OVERWRITE_DIFF_SIZE, PRESENCE_PARAM),
+            (self.PREVIEW, PRESENCE_PARAM),
             (self.CHECK, PRESENCE_PARAM),
             (self.QUIET, PRESENCE_PARAM),
+            (self.NO_HIDDEN, PRESENCE_PARAM),
+            (self.SYNC, PRESENCE_PARAM),
+
+            (self.MMAP, INT_PARAM),
+            (self.CHUNK_SIZE, INT_PARAM),
         ]
 
     @classmethod
@@ -2112,16 +2501,20 @@ Directories are automatically uploaded recursively.
 
 If a remote file has the same name of a local file, you will be asked \
 whether overwrite it or not. The default overwrite behaviour can be specified \
-with the options <b>-y</b> (yes), <b>-n</b> (no), <b>N</b> (newer)."""
-
+with the options <b>-y</b> (yes), <b>-n</b> (no), <b>-N</b> (overwrite if newer) 
+and <b>-S</b> (overwrite if size is different)."""
     @classmethod
     def options(cls) -> List[CommandOptionInfo]:
         return [
             CommandOptionInfo(cls.OVERWRITE_YES, "always overwrite files"),
             CommandOptionInfo(cls.OVERWRITE_NO, "never overwrite files"),
             CommandOptionInfo(cls.OVERWRITE_NEWER, "overwrite files only if newer"),
+            CommandOptionInfo(cls.OVERWRITE_DIFF_SIZE, "overwrite files only if size is different"),
+            CommandOptionInfo(cls.PREVIEW, "do not transfer, just show a preview of what will happen"),
             CommandOptionInfo(cls.CHECK, "performs a check of files consistency"),
             CommandOptionInfo(cls.QUIET, "doesn't show progress"),
+            CommandOptionInfo(cls.NO_HIDDEN, "doesn't copy hidden files"),
+            CommandOptionInfo(cls.SYNC, "synchronize (same as -N but remove old files)"),
         ]
 
     @classmethod
@@ -2298,6 +2691,7 @@ info <A> # just for alignment
 Show information of the remote server to which the connection is established.
 
 The reported information are: 
+- Server version
 - Server name
 - Server ip
 - Server port
@@ -2324,13 +2718,15 @@ Usage example:
 
 SERVER INFO
 
-Name:           alice-arch
-Address:        192.168.1.105
-Port:           12020
-Discoverable:   True
-Discover Port:  12019
-Auth:           False
-SSL:            True
+Name:             alice-arch
+Address:          192.168.1.105
+Port:             12020
+Discoverable:     True
+Discover Port:    12019
+Authentication:   False
+SSL:              True
+Remote execution: disabled
+Version:          0.5
 
 ================================
 
@@ -2435,6 +2831,9 @@ COMMANDS_INFO: Dict[str, Type[CommandInfo]] = {
     Commands.LOCAL_LIST_DIRECTORY: Ls,
     Commands.LOCAL_LIST_DIRECTORY_ENHANCED: L,
     Commands.LOCAL_TREE_DIRECTORY: Tree,
+    Commands.LOCAL_FIND: Find,
+    Commands.LOCAL_FIND_SHORT: Find,
+    Commands.LOCAL_DISK_USAGE: Du,
     Commands.LOCAL_CHANGE_DIRECTORY: Cd,
     Commands.LOCAL_CREATE_DIRECTORY: Mkdir,
     Commands.LOCAL_COPY: Cp,
@@ -2449,6 +2848,9 @@ COMMANDS_INFO: Dict[str, Type[CommandInfo]] = {
     Commands.REMOTE_LIST_DIRECTORY: Rls,
     Commands.REMOTE_LIST_DIRECTORY_ENHANCED: Rl,
     Commands.REMOTE_TREE_DIRECTORY: Rtree,
+    Commands.REMOTE_FIND: Rfind,
+    Commands.REMOTE_FIND_SHORT: Rfind,
+    Commands.REMOTE_DISK_USAGE: Rdu,
     Commands.REMOTE_CHANGE_DIRECTORY: Rcd,
     Commands.REMOTE_CREATE_DIRECTORY: Rmkdir,
     Commands.REMOTE_COPY: Rcp,
@@ -2463,12 +2865,13 @@ COMMANDS_INFO: Dict[str, Type[CommandInfo]] = {
     Commands.SCAN_SHORT: Scan,
 
     Commands.CONNECT: Connect,
+    Commands.CONNECT_SHORT: Connect,
+
     Commands.DISCONNECT: Disconnect,
 
     Commands.OPEN: Open,
     Commands.OPEN_SHORT: Open,
     Commands.CLOSE: Close,
-    Commands.CLOSE_SHORT: Close,
 
     Commands.GET: Get,
     Commands.GET_SHORT: Get,
