@@ -16,10 +16,10 @@ verbose, v          change verbosity level
 </i>
 <I+4>
 scan, s             scan the network for easyshare servers
-connect             connect to a remote server
+connect, c          connect to a remote server
 disconnect          disconnect from a remote server
 open, o             open a remote sharing (eventually discovering it)
-close, c            close the remote sharing
+close               close the remote sharing
 </i>
 <I+4>
 get, g              get files and directories from the remote sharing
@@ -35,6 +35,7 @@ mkdir               create a local directory
 cp                  copy files and directories locally
 mv                  move files and directories locally
 rm                  remove files and directories locally
+find                search for local files
 exec, :             execute an arbitrary command locally
 shell, sh           start a local shell
 </i>
@@ -48,6 +49,7 @@ rmkdir              create a remote directory
 rcp                 copy files and directories remotely
 rmv                 move files and directories remotely
 rrm                 remove files and directories remotely
+rfind               search for local files
 rexec, ::           execute an arbitrary command remotely
 rshell, rsh         start a remote shell
 </i>
@@ -62,7 +64,6 @@ Type <b>help <u>command</u> for the full documentation of a <u>command</u>.
 
 Available commands are:     
                     <a>
-<b>General commands</b>
 {_AVAILABLE_COMMANDS_STR}"""
 
 
