@@ -2,9 +2,19 @@ from os import stat_result
 from pathlib import Path
 from stat import S_ISDIR, S_ISREG
 
+from easyshare.utils.env import is_unix
+from easyshare.utils.types import itob
 from easyshare.logging import get_logger
 from easyshare.tree import TreeNodeDict
 
+
+if is_unix():
+    from pwd import getpwuid
+    from grp import getgrgid
+
+log = get_logger(__name__)
+
+
 # Use advanced typing if possible (Literal, TypedDict)
 # since offers a better developer experience
 # But since they have been implemented only from
@@ -15,7 +25,6 @@ from easyshare.tree import TreeNodeDict
 # ================= FILE INFO  ===================
 # ================================================
 
-log = get_logger(__name__)
 
 FTYPE_FILE = "file"
 FTYPE_DIR = "dir"
@@ -47,10 +56,13 @@ try:
 
     class FileInfo(TypedDict, total=False):
         """ Information of a file """
-        name: str
-        ftype: FileType
-        size: int
+        name: str           # usually the filename
+        ftype: FileType     # "file" or "dir"
+        size: int           # size in bytes
         mtime: int          # last modification time
+        perm: str           # permissions
+        user: str           # user owner
+        group: str          # group owner
 
     class FileInfoTreeNode(FileInfo, TreeNodeDict, total=False):
         pass
@@ -60,23 +72,71 @@ except:
     FileInfoNode = Dict[str, Union[str, FileType, int, List['FileInfoNode']]]
 
 
-def create_file_info(path: Path, stat: stat_result = None,
-                     name: str = None, raise_exceptions=False) -> Optional[FileInfo]:
+_users_cache = {}
+_groups_cache = {}
+
+def create_file_info(path: Path, *,
+                     fstat: stat_result = None,
+                     name: str = None,
+                     fetch_size: bool = True,
+                     fetch_time: bool = True,
+                     fetch_perm: bool = False,
+                     fetch_owner: bool = False,
+                     raise_exceptions: bool = False) -> Optional[FileInfo]:
     """
     Helper that creates a 'FileInfo' for the given path;
     'stat' can be given for avoid a stat() call.
     If 'name' is given, then it will be used instead of path.name.
     """
     try:
-        stat = stat or path.stat()
-        return {
+        fstat = fstat or path.stat()
+        finfo = {
             "name": name or path.name,
-            "ftype": ftype(path, stat),
-            "size": stat.st_size,
-            "mtime": stat.st_mtime_ns
+            "ftype": ftype(path, fstat),
         }
+
+        if fetch_size:
+            finfo["size"] = fstat.st_size
+        if fetch_time:
+            finfo["mtime"] = fstat.st_mtime_ns
+        if fetch_perm:
+            finfo["perm"] = oct(fstat.st_mode & 0o777)[-3:]
+
+        if fetch_owner:
+            user_name = _users_cache.get(fstat.st_uid)
+            if not user_name:
+                try:
+                    user_name = getpwuid(fstat.st_uid).pw_name
+                except:
+                    # Don't fail globally if user name can't be retrieved (permission problems?)
+                    # User the UID as fallback
+                    log.w("User name for UID %d can't be retrieved, "
+                          "using itself as fallback", fstat.st_uid)
+                    user_name = str(fstat.st_uid)
+                _users_cache[fstat.st_uid] = user_name
+                log.i(f"UID {fstat.st_gid} = '{user_name}'")
+
+            group_name = _groups_cache.get(fstat.st_gid)
+            if not group_name:
+                try:
+                    group_name = getgrgid(fstat.st_gid).gr_name
+                except:
+                    # Don't fail globally if group id can't be retrieved (permission problems?)
+                    # User the UID as fallback
+                    log.w("Group name for GID %d can't be retrieved, "
+                          "using itself as fallback", fstat.st_gid)
+                    group_name = str(fstat.st_gid)
+                _groups_cache[fstat.st_gid] = group_name
+                log.i(f"GID {fstat.st_gid} = '{group_name}'")
+
+            finfo["user"] = user_name
+            finfo["group"] = group_name
+
+
+        return finfo
+
     except Exception as ex:
-        log.w("Can't create file info - exception occurred")
+        log.w("Can't create file info - exception occurred: %s", str(ex))
         if raise_exceptions:
             raise ex
         return None
@@ -148,11 +208,6 @@ except:
 # ================================================
 
 
-class OverwritePolicy:
-    PROMPT = 0
-    YES = 1
-    NO = 2
-    NEWER = 3
 
 
 # ================================================
@@ -163,3 +218,20 @@ class PutNextResponse:
     ACCEPTED = "accepted"
     ASK_OVERWRITE = "ask_overwrite"
     REFUSED = "refused"
+
+
+class RexecEventType:
+    TEXT =      0
+    TEXT_B =    itob(TEXT, length=1)
+
+    RETCODE =   1
+    RETCODE_B = itob(RETCODE, length=1)
+
+    KILL =      2
+    KILL_B =    itob(KILL, length=1)
+
+    EOF =       3
+    EOF_B =     itob(EOF, length=1)
+
+    ENDACK =    255
+    ENDACK_B =  itob(ENDACK, length=1)
\ No newline at end of file
