@@ -2,11 +2,13 @@ import socket
 import ssl
 
 from abc import ABC
-from typing import Optional, Union, Tuple
+from typing import Optional, Union, Tuple, Callable
 
+from easyshare.common import TransferDirection, TransferProtocol
 from easyshare.consts.net import ADDR_BROADCAST, ADDR_ANY, PORT_ANY
 from easyshare.logging import get_logger
 from easyshare.endpoint import Endpoint
+from easyshare.tracing import trace_bin_all, trace_bin_payload
 from easyshare.utils.net import socket_udp_in, socket_udp_out, socket_tcp_out, socket_tcp_in
 from easyshare.utils.ssl import sslify_socket
 
@@ -27,7 +29,11 @@ class Socket(ABC):
         self.sock: Union[socket.socket, ssl.SSLSocket] = sock
 
     def endpoint(self) -> Endpoint:
-        return self.sock.getsockname()
+        try:
+            return self.sock.getsockname()
+        except:
+            log.exception("Cannot determinate remote endpoint")
+            return "0.0.0.0", 0  # fallback
 
     def address(self) -> str:
         return self.endpoint()[0]
@@ -61,10 +67,22 @@ class Socket(ABC):
 
 
 class SocketUdp(Socket):
-    def recv(self, bufsize=DEFAULT_SOCKET_BUFSIZE) -> Tuple[bytes, Endpoint]:
-        return self.sock.recvfrom(bufsize)
+    def recv(self, length=DEFAULT_SOCKET_BUFSIZE, trace: bool = True) -> Tuple[bytes, Endpoint]:
+        data, sender = self.sock.recvfrom(length)
+
+        if trace:
+            trace_bin_payload(data,
+                          sender=sender, receiver=self.endpoint(),
+                          direction=TransferDirection.IN, protocol=TransferProtocol.UDP)
+
+        return data, sender
+
+    def send(self, data: bytes, address: str, port: int, trace: bool = True) -> int:
+        if trace:
+            trace_bin_payload(data,
+                          sender=self.endpoint(), receiver=(address, port),
+                          direction=TransferDirection.OUT, protocol=TransferProtocol.UDP)
 
-    def send(self, data: bytes, address: str, port: int) -> int:
         return self.sock.sendto(data, (address, port))
 
     def broadcast(self, data: bytes, port: int) -> int:
@@ -89,18 +107,48 @@ class SocketUdpOut(SocketUdp):
 
 
 class SocketTcp(Socket):
-    def send(self, data: bytes):
+    def __init__(self, sock: socket.socket):
+        super().__init__(sock)
+        self._recv_buffer = bytearray()
+
+    def send(self, data: bytes, tracer: Callable = trace_bin_all):
+        if tracer:
+            tracer(data,
+                   sender=self.endpoint(), receiver=self.remote_endpoint(),
+                   direction=TransferDirection.OUT, protocol=TransferProtocol.TCP)
+
         self.sock.sendall(data)
 
-    def recv(self, bufsize=DEFAULT_SOCKET_BUFSIZE) -> bytes:
-        return self.sock.recv(bufsize)
+    def recv(self, length: int, tracer: Callable = trace_bin_all) -> Optional[bytearray]:
+        while True:
+            remaining_length = length - len(self._recv_buffer)
+            if remaining_length <= 0:
+                break
+
+            recvlen = min(remaining_length, DEFAULT_SOCKET_BUFSIZE)
+            data = self.sock.recv(recvlen)
+
+            if len(data) == 0:
+                return None
+
+            self._recv_buffer += data
+
+        data = self._recv_buffer[0:length]
+        self._recv_buffer = self._recv_buffer[length:]  # might have read more
+                                                        # than the required length
+        if tracer:
+            tracer(data,
+                   sender=self.remote_endpoint(), receiver=self.endpoint(),
+                   direction=TransferDirection.IN, protocol=TransferProtocol.TCP)
+
+        return data
 
     def remote_endpoint(self) -> Optional[Endpoint]:
         try:
             return self.sock.getpeername()
         except:
             log.exception("Cannot determinate remote endpoint")
-            return None
+            return "0.0.0.0", 0  # fallback
 
     def remote_address(self) -> str:
         return self.remote_endpoint()[0]
@@ -108,11 +156,13 @@ class SocketTcp(Socket):
     def remote_port(self) -> int:
         return self.remote_endpoint()[1]
 
+
 class SocketTcpIn(SocketTcp):
     def __init__(self,
                  sock: socket.socket):
         super().__init__(sock) # already sslified by the acceptor, eventually
 
+
 class SocketTcpOut(SocketTcp):
     def __init__(self,
                  address: str,
@@ -127,6 +177,7 @@ class SocketTcpOut(SocketTcp):
             )
         )
 
+
 class SocketTcpAcceptor(Socket):
 
     def __init__(self,
