@@ -1,41 +1,90 @@
-import fcntl
 import os
-import pty
+import re
 import select
 import shlex
 import shutil
 import subprocess
 import threading
-import tty
+from os import PathLike
 from pathlib import Path
-from pwd import getpwuid, struct_passwd
+from stat import S_ISREG
 from typing import Optional, List, Union, Tuple, Any, Callable
 
-from ptyprocess import PtyProcess, PtyProcessUnicode
-
 from easyshare.logging import get_logger
-from easyshare.protocol.types import FTYPE_DIR, FileInfoTreeNode, FileInfo, create_file_info
-from easyshare.utils.env import terminal_size
+from easyshare.protocol.types import FTYPE_DIR, FileInfoTreeNode, FileInfo, create_file_info, FileType
+from easyshare.utils.env import terminal_size, is_unix
 from easyshare.utils.path import is_hidden
+from easyshare.utils.str import isorted
 from easyshare.utils.types import list_wrap
 
 log = get_logger(__name__)
 
+_PERM_DIGIT_STR = {
+    "0": "---",
+    "1": "--x",
+    "2": "-w-",
+    "3": "-wx",
+    "4": "r--",
+    "5": "r-x",
+    "6": "rw-",
+    "7": "rwx",
+}
 
-def is_unix():
-    return os.name == "posix"
 
-def is_windows():
-    return os.name == "nt"
+if is_unix():
+    from pwd import getpwuid, struct_passwd
+    from grp import getgrgid, struct_group
+    from ptyprocess import PtyProcess, PtyProcessUnicode
+    import fcntl
+    import tty
+    import pty
+
+
+    def user(uid = os.geteuid()) -> struct_passwd:
+        """
+        Get the user entry for the given user id
+        (or the current one if not specified)
+        """
+        return getpwuid(uid)
+
+    def group(gid = os.getgid()) -> struct_group:
+
+        """
+        Get the group entry for the given group id
+        (or the current one if not specified)
+        """
+        return getgrgid(gid)
+else:
+    def user(uid):
+        raise ValueError("Not implemented")
+
+    def group(gid):
+        raise ValueError("Not implemented")
+
+
+def os_error_str(err: OSError):
+    """ Returns the explanation of the error (e.g. Directory not empty) """
+    if isinstance(err, OSError):
+        if err and err.strerror:
+            return err.strerror
+        log.exception("Unknown OS error")
+        serr = str(err)
+        return serr or "Error" # fallback
+    return "Error" # fallback
+
+
+def perm_str(perm: str):
+    return \
+        _PERM_DIGIT_STR.get(perm[0], "---") + \
+        _PERM_DIGIT_STR.get(perm[1], "---") + \
+        _PERM_DIGIT_STR.get(perm[2], "---")
 
-def get_passwd() -> struct_passwd:
-    """ Get the /etc/passwd entry for the current effective user id """
-    return getpwuid(os.geteuid())
 
 def ls(path: Path,
        sort_by: Union[str, List[str]] = "name",
        reverse: bool = False,
-       hidden: bool = False) -> Optional[List[FileInfo]]:
+       hidden: bool = False,
+       details: bool = False) -> List[FileInfo]:
     """ Wrapper of Path.iterdir() that provides a list of FileInfo """
 
     if not path:
@@ -51,7 +100,9 @@ def ls(path: Path,
     # Single file
     if path.is_file():
         # Show it even if it is hidden
-        finfo = create_file_info(path) # might fail (e.g. broken link)
+        finfo = create_file_info(path,
+                                 fetch_size=details, fetch_time=details,
+                                 fetch_perm=details, fetch_owner=details)
         if not finfo:
             return []
         return [finfo]
@@ -67,14 +118,15 @@ def ls(path: Path,
         if not hidden and is_hidden(p):
             log.d("Not showing hidden file: %s", p)
             continue
-
-        finfo = create_file_info(p) # might fail (e.g. broken link)
+        finfo = create_file_info(p,
+                                 fetch_size=details, fetch_time=details,
+                                 fetch_perm=details, fetch_owner=details)
         if finfo:
             ret.append(finfo)
 
     # Sort the result for each field of sort_by
     for sort_field in sort_by:
-        ret = sorted(ret, key=lambda fi: fi[sort_field])
+        ret = isorted(ret, key=lambda fi: fi[sort_field])
 
     if reverse:
         ret.reverse()
@@ -86,7 +138,8 @@ def tree(path: Path,
          sort_by: Union[str, List[str]] = "name",
          reverse: bool = False,
          max_depth: int = None,
-         hidden: bool = False) -> Optional[FileInfoTreeNode]:
+         hidden: bool = False,
+         details: bool = False) -> Optional[FileInfoTreeNode]:
     """
     Performs a traversal from the given 'path' and provide a 'FileInfoTreeNode'
     that represent the tree structure.
@@ -94,12 +147,16 @@ def tree(path: Path,
     if not path:
         raise TypeError("found invalid path")
 
+    path = path.resolve()
+
     sort_by = list(filter(lambda field: field in ["name", "size", "ftype"],
                           list_wrap(sort_by)))
 
-    log.i("TREE sorting by {}{}".format(sort_by, " (reverse)" if reverse else ""))
+    log.i("TREE on {}, sorting by {}{}".format(path, sort_by, " (reverse)" if reverse else ""))
 
-    root: Any = create_file_info(path) # might fail (e.g. broken link)
+    root: Any = create_file_info(path,
+                                 fetch_size=details, fetch_time=details,
+                                 fetch_perm=details, fetch_owner=details)
 
     if not root:
         return None
@@ -124,7 +181,8 @@ def tree(path: Path,
                 cursor["children_unseen_info"] = ls(cur_path,
                                                     sort_by=sort_by,
                                                     reverse=reverse,
-                                                    hidden=hidden)
+                                                    hidden=hidden,
+                                                    details=details)
             except OSError:
                 log.w("Cannot descend %s", cur_path)
                 pass
@@ -167,9 +225,10 @@ def tree(path: Path,
         child = dict(
             unseen_child_info,
             parent=cursor,
-            path=cur_path.joinpath(unseen_child_info.get("name"))
+            path=cur_path / unseen_child_info.get("name")
         )
 
+
         cursor.get("children").append(child)
 
         # Go down to this child
@@ -179,6 +238,138 @@ def tree(path: Path,
     return root
 
 
+def find(path: Union[Path, PathLike],
+         name: str = None,
+         regex: str = None,
+         case_sensitive: bool = True,
+         ftype: FileType = None,
+         details: bool = False,
+         file_info_name_provider: Callable[[Path], str] = str) -> Optional[List[FileInfo]]:
+
+    if not path:
+        raise TypeError("found invalid path")
+
+    log.i("FIND searching\n"
+          f"\tname={name}\n"
+          f"\tregex={regex}\n"
+          f"\tcase_sensitive={case_sensitive}\n"
+          f"\tftype={ftype}\n"
+          f"\tdetails={details}")
+
+    name_filter = None
+    regex_filter = None
+    ftype_filter = ftype
+
+    if name:
+        name_filter = name
+        if not case_sensitive:
+            name_filter = name_filter.lower()
+
+    if regex:
+        flags = 0
+        if not case_sensitive:
+            flags = re.IGNORECASE
+        try:
+            regex_filter = re.compile(regex, flags)
+            log.d("Regex compiled successfully")
+        except:
+            log.w("Invalid regex pattern: %s", regex)
+            return None
+
+    ret: List[FileInfo] = []
+
+    if not path.exists():
+        return ret
+
+    for f, fstat in walk_preorder(path):
+        p = Path(f)
+        finfo = create_file_info(p,
+                                 fstat=fstat,
+                                 name=file_info_name_provider(p),
+                                 fetch_size=details, fetch_time=details,
+                                 fetch_perm=details, fetch_owner=details)
+        if not finfo:
+            continue
+
+        log.d("finfo = %s", finfo)
+        f_name = finfo.get("name")
+
+        path_filter_subject = f_name
+
+        if case_sensitive is False:
+            path_filter_subject = path_filter_subject.lower()
+
+        # Check if satisfy the filter
+        if name_filter:
+            if name_filter not in path_filter_subject:
+                log.d("-> name filter failed")
+                continue
+        if regex_filter:
+            if not re.search(regex_filter, path_filter_subject):
+                log.d("-> regex filter failed")
+                continue
+
+        # Filename filters passed
+
+        # Type filters
+        ftype = finfo.get("ftype")
+        if ftype_filter and ftype_filter != ftype:
+            log.d("-> ftype filter failed")
+            continue
+
+        log.i("find ok: %s", f_name)
+        ret.append(finfo)
+
+
+    return ret
+
+
+def du(path: Path):
+
+    if not path:
+        raise TypeError("found invalid path")
+
+    log.i(f"DU {path}")
+
+    du_sum = 0
+
+    for f, fstat in walk_preorder(path):
+        du_sum += fstat.st_size
+
+    log.i(f"DU total: {du_sum}B")
+
+    return du_sum
+
+
+def walk_preorder(path: Path):
+    root = path
+    log.d("walk_preorder over '%s'", root)
+
+    stack: List[Path] = [root]
+
+    while stack:
+        cursor = stack.pop(0)
+
+        fstat = cursor.stat()
+
+        try:
+            is_file = S_ISREG(fstat.st_mode)
+        except OSError:
+            is_file = False
+
+        if is_file:
+            yield cursor, fstat
+        else:
+            if cursor != root:
+                yield cursor, fstat
+
+            try:
+                children: List = isorted(list(cursor.iterdir()))
+                stack = children + stack
+            except OSError as oserr:
+                log.w("Can't descend: %s", str(oserr))
+
+
 def rm(path: Path, error_callback: Callable[[Exception, Path], None] = None) -> bool:
     """
     Wrapper that remove path either if it is a file or a (even filled) directory.
@@ -235,13 +426,14 @@ def cp(src: Path, dest: Path):
     # we have to use copytree if we detect a DIR to DIR copy
     if src.is_dir() and dest.is_dir():
         log.d("Recursive copy DIR => DIR detected")
-        dest = dest.joinpath(src.name)
+        dest = dest / src.name
         log.d("Definitive src = '%s' | dst = '%s'", src, dest)
         shutil.copytree(str(src), str(dest))
     else:
         shutil.copy2(str(src), str(dest), follow_symlinks=False)
 
 
+
 def run_attached(cmd: str, stderr_redirect: int = None):
     """ Run a command while being attached to this terminal """
     proc = subprocess.Popen(cmd, shell=True, text=True, stderr=stderr_redirect)
@@ -258,7 +450,7 @@ def run_detached(cmd: str,
     The stdin can be provided writing on proc.stdin of this subprocess
     """
 
-    def proc_handler(proc: subprocess.Popen):
+    def out_handler(proc: subprocess.Popen):
         flags = fcntl.fcntl(proc.stdout, fcntl.F_GETFL)
         fcntl.fcntl(proc.stdout, fcntl.F_SETFL, flags | os.O_NONBLOCK)
 
@@ -289,10 +481,11 @@ def run_detached(cmd: str,
                                   stderr=subprocess.PIPE,
                                   stdin=subprocess.PIPE)
 
-    proc_handler_th = threading.Thread(target=proc_handler, daemon=True, args=(popen_proc, ))
-    proc_handler_th.start()
+    proc_out_handler_th = threading.Thread(
+        target=out_handler, daemon=True, args=(popen_proc, ))
+    proc_out_handler_th.start()
 
-    return popen_proc, proc_handler_th
+    return popen_proc, proc_out_handler_th
 
 def pty_attached(cmd: str = "/bin/sh") -> int:
     """
@@ -306,19 +499,21 @@ def pty_attached(cmd: str = "/bin/sh") -> int:
     pid, master_fd = pty.fork()
     if pid == pty.CHILD:
         os.execlp(argv[0], *argv)
+
+    tty_mode = None
     try:
-        mode = tty.tcgetattr(pty.STDIN_FILENO)
+        tty_mode = tty.tcgetattr(pty.STDIN_FILENO)
         tty.setraw(pty.STDIN_FILENO)
-        restore = 1
-    except tty.error:  # This is the same as termios.error
-        restore = 0
+    except tty.error:
+        pass
+
     try:
         pty._copy(master_fd, master_read, stdin_read)
-    except OSError:#
+    except OSError:
         pass
     finally:
-        if restore:
-            tty.tcsetattr(pty.STDIN_FILENO, tty.TCSAFLUSH, mode)
+        if tty_mode:
+            tty.tcsetattr(pty.STDIN_FILENO, tty.TCSAFLUSH, tty_mode)
 
     os.close(master_fd)
 
@@ -328,7 +523,7 @@ def pty_attached(cmd: str = "/bin/sh") -> int:
 
 def pty_detached(out_hook: Callable[[str], None],
                  end_hook: Callable[[int], None],
-                 cmd: str = "/bin/sh") -> PtyProcess:
+                 cmd: str = "/bin/sh"):  # -> PtyProcess:
     """
     Run a command, reporting stdout and stderr of the process outside (via out_hook).
     The stdin can be provided with ptyproc.write().
@@ -354,18 +549,4 @@ def pty_detached(out_hook: Callable[[str], None],
     proc_handler_th = threading.Thread(target=proc_handler, daemon=True)
     proc_handler_th.start()
 
-    return ptyproc
-
-
-def os_error_str(err: OSError):
-    """ Returns the explanation of the error (e.g. Directory not empty) """
-    if isinstance(err, OSError):
-        if err and err.strerror:
-            return err.strerror
-        log.exception("Unknown OS error")
-        serr = str(err)
-        return serr or "Error" # fallback
-    return "Error" # fallback
-
-if __name__ == "__main__":
-    print("OS: ", "windows" if is_windows() else ("unix" if is_unix() else "unknown"))
\ No newline at end of file
+    return ptyproc
\ No newline at end of file
