@@ -1,7 +1,7 @@
 import pkgutil
 
 from easyshare.logging import get_logger
-from easyshare.utils.types import bytes_to_str
+from easyshare.utils.types import btos
 
 log = get_logger(__name__)
 
@@ -12,4 +12,4 @@ def read_resource_string(pkg: str, res: str) -> str:
     if not b:
         log.w("Failed to load resource '%s'", res)
         return ""
-    return bytes_to_str(b)
+    return btos(b)
